@@ -34,10 +34,8 @@ else
 fi
 ZOOBINDIR=`dirname "$ZOOBIN"`
 
-. $ZOOBINDIR/zkEnv.sh
-
-eval `grep -e "^dataDir=" $ZOOCFG`
+. "$ZOOBINDIR"/zkEnv.sh
 
 java "-Dzookeeper.log.dir=${ZOO_LOG_DIR}" "-Dzookeeper.root.logger=${ZOO_LOG4J_PROP}" \
-     -cp $CLASSPATH $JVMFLAGS \
+     -cp "$CLASSPATH" $JVMFLAGS \
      org.apache.zookeeper.ZooKeeperMain $@
