@@ -18,9 +18,9 @@
 
 package org.apache.zookeeper.server;
 
-import java.io.ByteArrayOutputStream;
 import java.io.File;
 import java.io.IOException;
+import java.io.PrintWriter;
 import java.nio.ByteBuffer;
 import java.util.ArrayList;
 import java.util.Arrays;
@@ -29,12 +29,8 @@ import java.util.LinkedList;
 import java.util.List;
 import java.util.Random;
 import java.util.concurrent.ConcurrentHashMap;
-import java.util.concurrent.LinkedBlockingQueue;
 
 import org.apache.jute.BinaryInputArchive;
-import org.apache.jute.BinaryOutputArchive;
-import org.apache.jute.InputArchive;
-import org.apache.jute.OutputArchive;
 import org.apache.jute.Record;
 import org.apache.log4j.Logger;
 import org.apache.zookeeper.Environment;
@@ -45,14 +41,9 @@ import org.apache.zookeeper.data.Id;
 import org.apache.zookeeper.data.StatPersisted;
 import org.apache.zookeeper.jmx.MBeanRegistry;
 import org.apache.zookeeper.proto.RequestHeader;
+import org.apache.zookeeper.server.SessionTracker.Session;
 import org.apache.zookeeper.server.SessionTracker.SessionExpirer;
 import org.apache.zookeeper.server.persistence.FileTxnSnapLog;
-import org.apache.zookeeper.server.persistence.FileTxnSnapLog.PlayBackListener;
-import org.apache.zookeeper.server.quorum.Leader;
-import org.apache.zookeeper.server.quorum.QuorumPacket;
-import org.apache.zookeeper.server.quorum.Leader.Proposal;
-import org.apache.zookeeper.server.util.SerializeUtils;
-import org.apache.zookeeper.txn.TxnHeader;
 
 /**
  * This class implements a simple standalone ZooKeeperServer. It sets up the
@@ -96,18 +87,16 @@ public class ZooKeeperServer implements SessionExpirer, ServerStats.Provider {
 
     public static final int DEFAULT_TICK_TIME = 3000;
     protected int tickTime = DEFAULT_TICK_TIME;
-
-    public static final int commitLogCount = 500;
-    public int commitLogBuffer = 700;
-    public LinkedList<Proposal> committedLog = new LinkedList<Proposal>();
-    public long minCommittedLog, maxCommittedLog;
-    private DataTreeBuilder treeBuilder;
-    public DataTree dataTree;
+    /** value of -1 indicates unset, use default */
+    protected int minSessionTimeout = -1;
+    /** value of -1 indicates unset, use default */
+    protected int maxSessionTimeout = -1;
     protected SessionTracker sessionTracker;
     private FileTxnSnapLog txnLogFactory = null;
-    protected ConcurrentHashMap<Long, Integer> sessionsWithTimeouts;
+    private ConcurrentHashMap<Long, Integer> sessionsWithTimeouts;
+    private ZKDatabase zkDb;
     protected long hzxid = 0;
-    final public static Exception ok = new Exception("No prob");
+    public final static Exception ok = new Exception("No prob");
     protected RequestProcessor firstProcessor;
     protected volatile boolean running;
 
@@ -118,16 +107,17 @@ public class ZooKeeperServer implements SessionExpirer, ServerStats.Provider {
     static final private long superSecret = 0XB3415C00L;
 
     int requestsInProcess;
-    List<ChangeRecord> outstandingChanges = new ArrayList<ChangeRecord>();
+    final List<ChangeRecord> outstandingChanges = new ArrayList<ChangeRecord>();
     // this data structure must be accessed under the outstandingChanges lock
-    HashMap<String, ChangeRecord> outstandingChangesForPath = new HashMap<String, ChangeRecord>();
+    final HashMap<String, ChangeRecord> outstandingChangesForPath =
+        new HashMap<String, ChangeRecord>();
     
     private NIOServerCnxn.Factory serverCnxnFactory;
 
     private final ServerStats serverStats;
 
     void removeCnxn(ServerCnxn cnxn) {
-        dataTree.removeCnxn(cnxn);
+        zkDb.removeCnxn(cnxn);
     }
  
     /**
@@ -139,31 +129,69 @@ public class ZooKeeperServer implements SessionExpirer, ServerStats.Provider {
      */
     public ZooKeeperServer() {
         serverStats = new ServerStats(this);
-        treeBuilder = new BasicDataTreeBuilder();
     }
     
     /**
      * Creates a ZooKeeperServer instance. It sets everything up, but doesn't
      * actually start listening for clients until run() is invoked.
-     *
+     * 
      * @param dataDir the directory to put the data
      * @throws IOException
      */
     public ZooKeeperServer(FileTxnSnapLog txnLogFactory, int tickTime,
-            DataTreeBuilder treeBuilder) throws IOException {
+            int minSessionTimeout, int maxSessionTimeout,
+            DataTreeBuilder treeBuilder, ZKDatabase zkDb) throws IOException {
         serverStats = new ServerStats(this);
-        this.treeBuilder = treeBuilder;
-
         this.txnLogFactory = txnLogFactory;
+        this.zkDb = zkDb;
         this.tickTime = tickTime;
+        this.minSessionTimeout = minSessionTimeout;
+        this.maxSessionTimeout = maxSessionTimeout;
         
-        LOG.info("Created server");
+        LOG.info("Created server with tickTime " + tickTime
+                + " minSessionTimeout " + getMinSessionTimeout()
+                + " maxSessionTimeout " + getMaxSessionTimeout()
+                + " datadir " + txnLogFactory.getDataDir()
+                + " snapdir " + txnLogFactory.getSnapDir());
     }
 
+    /**
+     * creates a zookeeperserver instance. 
+     * @param txnLogFactory the file transaction snapshot logging class
+     * @param tickTime the ticktime for the server
+     * @param treeBuilder the datatree builder
+     * @throws IOException
+     */
+    public ZooKeeperServer(FileTxnSnapLog txnLogFactory, int tickTime,
+            DataTreeBuilder treeBuilder) throws IOException {
+        this(txnLogFactory, tickTime, -1, -1, treeBuilder,
+                new ZKDatabase(txnLogFactory));
+    }
+    
     public ServerStats serverStats() {
         return serverStats;
     }
-    
+
+    public void dumpConf(PrintWriter pwriter) {
+        pwriter.print("clientPort=");
+        pwriter.println(getClientPort());
+        pwriter.print("dataDir=");
+        pwriter.println(zkDb.snapLog.getSnapDir().getAbsolutePath());
+        pwriter.print("dataLogDir=");
+        pwriter.println(zkDb.snapLog.getDataDir().getAbsolutePath());
+        pwriter.print("tickTime=");
+        pwriter.println(getTickTime());
+        pwriter.print("maxClientCnxns=");
+        pwriter.println(serverCnxnFactory.getMaxClientCnxns());
+        pwriter.print("minSessionTimeout=");
+        pwriter.println(getMinSessionTimeout());
+        pwriter.print("maxSessionTimeout=");
+        pwriter.println(getMaxSessionTimeout());
+
+        pwriter.print("serverId=");
+        pwriter.println(getServerId());
+    }
+
     /**
      * This constructor is for backward compatibility with the existing unit
      * test code.
@@ -171,8 +199,8 @@ public class ZooKeeperServer implements SessionExpirer, ServerStats.Provider {
      */
     public ZooKeeperServer(File snapDir, File logDir, int tickTime)
             throws IOException {
-        this(new FileTxnSnapLog(snapDir,logDir),
-                tickTime,new BasicDataTreeBuilder());
+        this( new FileTxnSnapLog(snapDir, logDir),
+                tickTime, new BasicDataTreeBuilder());
     }
 
     /**
@@ -180,85 +208,56 @@ public class ZooKeeperServer implements SessionExpirer, ServerStats.Provider {
      *
      * @throws IOException
      */
-    public ZooKeeperServer(FileTxnSnapLog txnLogFactory,DataTreeBuilder treeBuilder) throws IOException {
-        this(txnLogFactory, DEFAULT_TICK_TIME, treeBuilder);
+    public ZooKeeperServer(FileTxnSnapLog txnLogFactory,
+            DataTreeBuilder treeBuilder)
+        throws IOException
+    {
+        this(txnLogFactory, DEFAULT_TICK_TIME, -1, -1, treeBuilder,
+                new ZKDatabase(txnLogFactory));
     }
 
+    /**
+     * get the zookeeper database for this server
+     * @return the zookeeper database for this server
+     */
+    public ZKDatabase getZKDatabase() {
+        return this.zkDb;
+    }
+    
+    /**
+     * set the zkdatabase for this zookeeper server
+     * @param zkDb
+     */
+    public void setZKDatabase(ZKDatabase zkDb) {
+       this.zkDb = zkDb;
+    }
+    
     /**
      *  Restore sessions and data
      */
     public void loadData() throws IOException, InterruptedException {
-        PlayBackListener listener=new PlayBackListener(){
-            public void onTxnLoaded(TxnHeader hdr,Record txn){
-                Request r = new Request(null, 0, hdr.getCxid(),hdr.getType(),
-                        null, null);
-                r.txn = txn;
-                r.hdr = hdr;
-                r.zxid = hdr.getZxid();
-                addCommittedProposal(r);
-            }
-        };
-        sessionsWithTimeouts = new ConcurrentHashMap<Long, Integer>();
-        dataTree = treeBuilder.build();
-        setZxid(txnLogFactory.restore(dataTree,sessionsWithTimeouts,listener));
+        zkDb.loadDataBase();
+        setZxid(zkDb.loadDataBase());
         // Clean up dead sessions
         LinkedList<Long> deadSessions = new LinkedList<Long>();
-        for (long session : dataTree.getSessions()) {
+        for (long session : zkDb.getSessions()) {
+            sessionsWithTimeouts = zkDb.getSessionWithTimeOuts();
             if (sessionsWithTimeouts.get(session) == null) {
                 deadSessions.add(session);
             }
         }
-        dataTree.initialized = true;
+        zkDb.setDataTreeInit(true);
         for (long session : deadSessions) {
             // XXX: Is lastProcessedZxid really the best thing to use?
-            killSession(session, dataTree.lastProcessedZxid);
+            killSession(session, zkDb.getDataTreeLastProcessedZxid());
         }
         // Make a clean snapshot
         takeSnapshot();
     }
 
-    /**
-     * maintains a list of last 500 or so committed requests. This is used for
-     * fast follower synchronization.
-     *
-     * @param request committed request
-     */
-
-    public void addCommittedProposal(Request request) {
-        synchronized (committedLog) {
-            if (committedLog.size() > commitLogCount) {
-                committedLog.removeFirst();
-                minCommittedLog = committedLog.getFirst().packet.getZxid();
-            }
-            if (committedLog.size() == 0) {
-                minCommittedLog = request.zxid;
-                maxCommittedLog = request.zxid;
-            }
-
-            ByteArrayOutputStream baos = new ByteArrayOutputStream();
-            BinaryOutputArchive boa = BinaryOutputArchive.getArchive(baos);
-            try {
-                request.hdr.serialize(boa, "hdr");
-                if (request.txn != null) {
-                    request.txn.serialize(boa, "txn");
-                }
-                baos.close();
-            } catch (IOException e) {
-                LOG.error("This really should be impossible", e);
-            }
-            QuorumPacket pp = new QuorumPacket(Leader.PROPOSAL, request.zxid,
-                    baos.toByteArray(), null);
-            Proposal p = new Proposal();
-            p.packet = pp;
-            p.request = request;
-            committedLog.add(p);
-            maxCommittedLog = p.packet.getZxid();
-        }
-    }
-
     public void takeSnapshot(){
         try {
-            txnLogFactory.save(dataTree, sessionsWithTimeouts);
+            txnLogFactory.save(zkDb.getDataTree(), zkDb.getSessionWithTimeOuts());
         } catch (IOException e) {
             LOG.fatal("Severe unrecoverable error, exiting", e);
             // This is a severe error that we cannot recover from,
@@ -267,18 +266,7 @@ public class ZooKeeperServer implements SessionExpirer, ServerStats.Provider {
         }
     }
 
-    public void serializeSnapshot(OutputArchive oa) throws IOException,
-            InterruptedException {
-        SerializeUtils.serializeSnapshot(dataTree, oa, sessionsWithTimeouts);
-    }
-
-    public void deserializeSnapshot(InputArchive ia) throws IOException {
-        sessionsWithTimeouts = new ConcurrentHashMap<Long, Integer>();
-        dataTree = treeBuilder.build();
-
-        SerializeUtils.deserializeSnapshot(dataTree,ia,sessionsWithTimeouts);
-    }
-
+  
     /**
      * This should be called from a synchronized block on this!
      */
@@ -311,7 +299,7 @@ public class ZooKeeperServer implements SessionExpirer, ServerStats.Provider {
     }
 
     protected void killSession(long sessionId, long zxid) {
-        dataTree.killSession(sessionId, zxid);
+        zkDb.killSession(sessionId, zxid);
         if (LOG.isTraceEnabled()) {
             ZooTrace.logTraceMessage(LOG, ZooTrace.SESSION_TRACE_MASK,
                                          "ZooKeeperServer --- killSession: 0x"
@@ -322,19 +310,31 @@ public class ZooKeeperServer implements SessionExpirer, ServerStats.Provider {
         }
     }
 
-    public void expire(long sessionId) {
-        LOG.info("Expiring session 0x" + Long.toHexString(sessionId));
+    public void expire(Session session) {
+        long sessionId = session.getSessionId();
+        LOG.info("Expiring session 0x" + Long.toHexString(sessionId)
+                + ", timeout of " + session.getTimeout() + "ms exceeded");
         close(sessionId);
     }
 
-    void touch(ServerCnxn cnxn) throws IOException {
+    public static class MissingSessionException extends IOException {
+        private static final long serialVersionUID = 7467414635467261007L;
+
+        public MissingSessionException(String msg) {
+            super(msg);
+        }
+    }
+    
+    void touch(ServerCnxn cnxn) throws MissingSessionException {
         if (cnxn == null) {
             return;
         }
         long id = cnxn.getSessionId();
         int to = cnxn.getSessionTimeout();
         if (!sessionTracker.touchSession(id, to)) {
-            throw new IOException("Missing session 0x" + Long.toHexString(id));
+            throw new MissingSessionException(
+                    "No session with sessionid 0x" + Long.toHexString(id)
+                    + " exists, probably expired and removed");
         }
     }
 
@@ -345,7 +345,7 @@ public class ZooKeeperServer implements SessionExpirer, ServerStats.Provider {
             MBeanRegistry.getInstance().register(jmxServerBean, null);
             
             try {
-                jmxDataTreeBean = new DataTreeBean(dataTree);
+                jmxDataTreeBean = new DataTreeBean(zkDb.getDataTree());
                 MBeanRegistry.getInstance().register(jmxDataTreeBean, jmxServerBean);
             } catch (Exception e) {
                 LOG.warn("Failed to register with JMX", e);
@@ -358,7 +358,11 @@ public class ZooKeeperServer implements SessionExpirer, ServerStats.Provider {
     }
     
     public void startup() throws IOException, InterruptedException {
-        if (dataTree == null) {
+        //check to see if zkDb is not null
+        if (zkDb == null) {
+            zkDb = new ZKDatabase(this.txnLogFactory);
+        }
+        if (!zkDb.isInitialized()) {
             loadData();
         }
         createSessionTracker();
@@ -382,7 +386,7 @@ public class ZooKeeperServer implements SessionExpirer, ServerStats.Provider {
     }
 
     protected void createSessionTracker() {
-        sessionTracker = new SessionTrackerImpl(this, sessionsWithTimeouts,
+        sessionTracker = new SessionTrackerImpl(this, zkDb.getSessionWithTimeOuts(),
                 tickTime, 1);
         ((SessionTrackerImpl)sessionTracker).start();
     }
@@ -403,8 +407,8 @@ public class ZooKeeperServer implements SessionExpirer, ServerStats.Provider {
         if (firstProcessor != null) {
             firstProcessor.shutdown();
         }
-        if (dataTree != null) {
-            dataTree.clear();
+        if (zkDb != null) {
+            zkDb.clear();
         }
 
         unregisterJMX();
@@ -578,8 +582,10 @@ public class ZooKeeperServer implements SessionExpirer, ServerStats.Provider {
                 LOG.warn("Dropping packet at server of type " + si.type);
                 // if invalid packet drop the packet.
             }
-        } catch (IOException e) {
-            LOG.warn("Ignoring unexpected exception", e);
+        } catch (MissingSessionException e) {
+            if (LOG.isDebugEnabled()) {
+                LOG.debug("Dropping request: " + e.getMessage());
+            }
         }
     }
 
@@ -623,7 +629,7 @@ public class ZooKeeperServer implements SessionExpirer, ServerStats.Provider {
      * datatree
      */
     public long getLastProcessedZxid() {
-        return dataTree.lastProcessedZxid;
+        return zkDb.getDataTreeLastProcessedZxid();
     }
 
     /**
@@ -643,33 +649,36 @@ public class ZooKeeperServer implements SessionExpirer, ServerStats.Provider {
      * @throws IOException
      */
     public void truncateLog(long zxid) throws IOException {
-        this.txnLogFactory.truncateLog(zxid);
+        this.zkDb.truncateLog(zxid);
     }
-    
-    /**
-     * the snapshot and logwriter for this instance
-     * @return
-     */
-    public FileTxnSnapLog getLogWriter() {
-        return this.txnLogFactory;
-    }
-    
+       
     public int getTickTime() {
         return tickTime;
     }
 
     public void setTickTime(int tickTime) {
+        LOG.info("tickTime set to " + tickTime);
         this.tickTime = tickTime;
     }
 
-    public DataTreeBuilder getTreeBuilder() {
-        return treeBuilder;
+    public int getMinSessionTimeout() {
+        return minSessionTimeout == -1 ? tickTime * 2 : minSessionTimeout;
     }
 
-    public void setTreeBuilder(DataTreeBuilder treeBuilder) {
-        this.treeBuilder = treeBuilder;
+    public void setMinSessionTimeout(int min) {
+        LOG.info("minSessionTimeout set to " + min);
+        this.minSessionTimeout = min;
     }
-    
+
+    public int getMaxSessionTimeout() {
+        return maxSessionTimeout == -1 ? tickTime * 20 : maxSessionTimeout;
+    }
+
+    public void setMaxSessionTimeout(int max) {
+        LOG.info("maxSessionTimeout set to " + max);
+        this.maxSessionTimeout = max;
+    }
+
     public int getClientPort() {
         return serverCnxnFactory != null ? serverCnxnFactory.ss.socket().getLocalPort() : -1;
     }
@@ -685,4 +694,9 @@ public class ZooKeeperServer implements SessionExpirer, ServerStats.Provider {
     public String getState() {
         return "standalone";
     }
+
+    public void dumpEphemerals(PrintWriter pwriter) {
+    	zkDb.dumpEphemerals(pwriter);
+    }
+    
 }
